package com.perforce.p4simulink;

/**
 * Perforce SCM Integration
 */

import com.mathworks.cmlink.api.ApplicationInteractor;
import com.mathworks.cmlink.api.ConfigurationManagementException;
import com.mathworks.cmlink.api.InteractorSupportedFeature;
import com.mathworks.cmlink.api.customization.CustomizationWidgetFactory;
import com.mathworks.cmlink.api.version.r14a.CMInteractor;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.AccessException;
import com.perforce.p4java.exception.ConnectionException;
import com.perforce.p4java.exception.RequestException;
import com.perforce.p4java.impl.generic.core.file.FileSpec;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4simulink.connection.Config;
import com.perforce.p4simulink.connection.Connection;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.util.*;

public class P4Interactor implements CMInteractor {
    //	List of supported features.
    private static final Collection<InteractorSupportedFeature> supportedFeatures = EnumSet.of(
            InteractorSupportedFeature.CONNECTION
    );
    private static final Logger log = LogManager.getLogger(P4Interactor.class.getName());
    protected Config config;
    protected Connection connection;

    public P4Interactor(ApplicationInteractor applicationInteractor, Config config) throws P4CMException {
        this.config = config;
        //this.config.load();
        this.connection = new Connection();
    }

    @Override
    public void buildCustomActions(CustomizationWidgetFactory customizationWidgetFactory) {
        log.debug("buildCustomActions()");
    }

    @Override
    public boolean isFeatureSupported(InteractorSupportedFeature interactorSupportedFeature) {
        return supportedFeatures.contains(interactorSupportedFeature);
    }

    /**
     * Checks if the integration is ready.  As the integration needs to be connected to a Perforce Server,
     * we check if a IOptionsServer object exists for the session and is connected.  If it does,
     * then we're ready; otherwise, we still need to connect.
     *
     * @return is the connection to the Perforce Server ready or not.
     */
    @Override
    public boolean isReady() {
        return connection.isReady();
    }

    /**
     * Disconnect from the Perforce Server and close the Perforce session.
     *
     * @throws ConfigurationManagementException
     */
    @Override
    public void disconnect() throws ConfigurationManagementException {
        connection.disconnect();
    }

    /**
     *
     * @param clientName
     * @return IClient  client
     */
    public IClient getClient(String clientName) throws ConfigurationManagementException {
        try {
            return getServer().getClient(clientName);
        } catch (ConnectionException e) {
            logException(e);
        } catch (RequestException e) {
            logException(e);
        } catch (AccessException e) {
            logException(e);
        }
        return null;
    }

    public IClient getClient() throws ConfigurationManagementException {
        return getClient(config.getClient());
    }

    public IOptionsServer getServer() {
        return connection.getServer();
    }

    public List<IFileSpec> toSpec(Collection<File> files) {
        List<IFileSpec> specs = new ArrayList<>();
        for (File file : files) {
            specs.add(new FileSpec(file.getAbsolutePath()));
        }
        return specs;
    }

    public List<IFileSpec> toSpec(File file) {
        List<IFileSpec> specs = new ArrayList<>();
        specs.add(new FileSpec(file.getAbsolutePath()));
        return specs;
    }

    public Map<String, IFileSpec> toMap(List<IFileSpec> specs) {
        Map<String, IFileSpec> pathMap = new HashMap<>();
        for(IFileSpec spec : specs) {
            pathMap.put(spec.getLocalPathString(), spec);
        }
        return pathMap;
    }

    public Collection<File> toFileList(File file) {
        Collection<File> files = new HashSet<>();
        files.add(file);
        return files;
    }

    /**
     * Get the address from our P4Config object and attempt to connect to that endpoint.  If there are any problems
     * connecting to the server, catch the exception and rethrow as a P4CMException.
     *
     * @throws ConfigurationManagementException
     */
    @Override
    public void connect() throws ConfigurationManagementException {
    	System.out.println(config.toString());
        connection.connect(config);
    }

    @Override
    public String getSystemName() {
        return "Perforce SCM Integration for Simulink";
    }

    @Override
    public String getShortSystemName() {
        return "P4Simulink";
    }

    private void logException(Exception exception) throws ConfigurationManagementException {
        // log the error message
        String message = exception.getMessage();
        log.error(message);
        // print a stack trace to the debug log
        exception.printStackTrace(log.getStream(Level.getLevel("debug")));
        throw new ConfigurationManagementException(message);
    }
}
